from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ResponseHeader(_message.Message):
    __slots__ = ("success", "cloud_synced", "response_code", "response_message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    CLOUD_SYNCED_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_CODE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    cloud_synced: bool
    response_code: int
    response_message: str
    def __init__(self, success: bool = ..., cloud_synced: bool = ..., response_code: _Optional[int] = ..., response_message: _Optional[str] = ...) -> None: ...

class MessageDetails(_message.Message):
    __slots__ = ("message_id", "channel_name", "payload", "timestamp")
    MESSAGE_ID_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_NAME_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    message_id: str
    channel_name: str
    payload: str
    timestamp: str
    def __init__(self, message_id: _Optional[str] = ..., channel_name: _Optional[str] = ..., payload: _Optional[str] = ..., timestamp: _Optional[str] = ...) -> None: ...

class ChannelDetails(_message.Message):
    __slots__ = ("channel_name", "aggregate")
    CHANNEL_NAME_FIELD_NUMBER: _ClassVar[int]
    AGGREGATE_FIELD_NUMBER: _ClassVar[int]
    channel_name: str
    aggregate: str
    def __init__(self, channel_name: _Optional[str] = ..., aggregate: _Optional[str] = ...) -> None: ...

class AgentDetails(_message.Message):
    __slots__ = ("agent_id", "agent_name", "channels")
    AGENT_ID_FIELD_NUMBER: _ClassVar[int]
    AGENT_NAME_FIELD_NUMBER: _ClassVar[int]
    CHANNELS_FIELD_NUMBER: _ClassVar[int]
    agent_id: str
    agent_name: str
    channels: _containers.RepeatedCompositeFieldContainer[ChannelDetails]
    def __init__(self, agent_id: _Optional[str] = ..., agent_name: _Optional[str] = ..., channels: _Optional[_Iterable[_Union[ChannelDetails, _Mapping]]] = ...) -> None: ...

class TestCommsRequest(_message.Message):
    __slots__ = ("message",)
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    message: str
    def __init__(self, message: _Optional[str] = ...) -> None: ...

class TestCommsResponse(_message.Message):
    __slots__ = ("response_header", "response")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    response: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., response: _Optional[str] = ...) -> None: ...

class MessageDetailsRequest(_message.Message):
    __slots__ = ("message_id",)
    MESSAGE_ID_FIELD_NUMBER: _ClassVar[int]
    message_id: str
    def __init__(self, message_id: _Optional[str] = ...) -> None: ...

class MessageDetailsResponse(_message.Message):
    __slots__ = ("response_header", "message")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    message: MessageDetails
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., message: _Optional[_Union[MessageDetails, _Mapping]] = ...) -> None: ...

class ChannelDetailsRequest(_message.Message):
    __slots__ = ("channel_name",)
    CHANNEL_NAME_FIELD_NUMBER: _ClassVar[int]
    channel_name: str
    def __init__(self, channel_name: _Optional[str] = ...) -> None: ...

class ChannelDetailsResponse(_message.Message):
    __slots__ = ("response_header", "channel")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    channel: ChannelDetails
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., channel: _Optional[_Union[ChannelDetails, _Mapping]] = ...) -> None: ...

class ChannelSubscriptionRequest(_message.Message):
    __slots__ = ("channel_name",)
    CHANNEL_NAME_FIELD_NUMBER: _ClassVar[int]
    channel_name: str
    def __init__(self, channel_name: _Optional[str] = ...) -> None: ...

class ChannelSubscriptionResponse(_message.Message):
    __slots__ = ("response_header", "channel", "message")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    channel: ChannelDetails
    message: MessageDetails
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., channel: _Optional[_Union[ChannelDetails, _Mapping]] = ..., message: _Optional[_Union[MessageDetails, _Mapping]] = ...) -> None: ...

class ChannelWriteRequest(_message.Message):
    __slots__ = ("channel_name", "message_payload", "save_log")
    CHANNEL_NAME_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    SAVE_LOG_FIELD_NUMBER: _ClassVar[int]
    channel_name: str
    message_payload: str
    save_log: bool
    def __init__(self, channel_name: _Optional[str] = ..., message_payload: _Optional[str] = ..., save_log: bool = ...) -> None: ...

class DebugInfoRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ChannelWriteResponse(_message.Message):
    __slots__ = ("response_header", "message_id")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_ID_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    message_id: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., message_id: _Optional[str] = ...) -> None: ...

class AgentDetailsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class AgentDetailsResponse(_message.Message):
    __slots__ = ("response_header", "agent")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    AGENT_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    agent: AgentDetails
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., agent: _Optional[_Union[AgentDetails, _Mapping]] = ...) -> None: ...

class DebugInfoResponse(_message.Message):
    __slots__ = ("response_header", "active_callbacks", "wss_callbacks", "wss_aggregates", "wss_subscriptions")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_CALLBACKS_FIELD_NUMBER: _ClassVar[int]
    WSS_CALLBACKS_FIELD_NUMBER: _ClassVar[int]
    WSS_AGGREGATES_FIELD_NUMBER: _ClassVar[int]
    WSS_SUBSCRIPTIONS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    active_callbacks: _containers.RepeatedScalarFieldContainer[str]
    wss_callbacks: _containers.RepeatedScalarFieldContainer[str]
    wss_aggregates: _containers.RepeatedScalarFieldContainer[str]
    wss_subscriptions: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., active_callbacks: _Optional[_Iterable[str]] = ..., wss_callbacks: _Optional[_Iterable[str]] = ..., wss_aggregates: _Optional[_Iterable[str]] = ..., wss_subscriptions: _Optional[_Iterable[str]] = ...) -> None: ...

class TempAPITokenRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class TempAPITokenResponse(_message.Message):
    __slots__ = ("response_header", "token", "valid_until", "endpoint")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    VALID_UNTIL_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    token: str
    valid_until: str
    endpoint: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., token: _Optional[str] = ..., valid_until: _Optional[str] = ..., endpoint: _Optional[str] = ...) -> None: ...



import asyncio
import json
import logging
import time

from typing import Optional, Any

DEFAULT_WAKE_DELAY = 5


class SnapshotInProgress(Exception):
    def __init__(self):
        super().__init__("There is already a snapshot in progress for this camera.")


class PowerContext:
    def __init__(self, camera_rtsp: str, manager: "CameraPowerManagement"):
        self.camera = camera_rtsp
        self.manager = manager

    async def __aenter__(self):
        if not self.manager.plt_iface:
            # early exit if we don't define the platform interface (testing, etc.)
            return

        await self.manager.power_on()
        if not self.manager.is_powered:
            return  # something went wrong with trying to turn the power on...

        wake_delay = await self.manager.get_wake_delay(self.camera)
        to_sleep = wake_delay - (time.time() - self.manager.start_powered)
        if to_sleep > 0:
            await asyncio.sleep(to_sleep)

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.manager.handle_cam_done(self.camera)


class CameraPowerManagement:
    def __init__(self, plt_iface=None, config_manager=None, default_config=None):
        self.plt_iface = plt_iface
        self.config_manager = config_manager
        self.default_config = default_config

        self.is_powered: bool = False
        self.start_powered = None

        self.active_cameras = set()

    async def get_config(self) -> Optional[dict[str, Any]]:
        if not self.config_manager and not self.default_config:
            logging.warning("No config_manager or default_config supplied.")
            return
        elif not self.config_manager:
            config = self.default_config
        else:
            config = await self.config_manager.get_config_async('camera_config')

        if not config:
            logging.warning("No camera config found.")
            return

        if isinstance(config, str):
            try:
                config = json.loads(config)
            except json.JSONDecodeError:
                logging.warning("Unable to parse camera config as valid JSON from config manager")
                return config  # might still be a valid string?

        return config

    async def get_power_pin(self) -> Optional[int]:
        config = await self.get_config()
        if not config:
            return

        try:
            return config['POWER_PIN']
        except KeyError:
            logging.warning("Unable to parse camera power pin from config")

        return None

    async def get_wake_delay(self, rtsp_uri) -> Optional[int]:
        config = await self.get_config()
        if not config:
            return DEFAULT_WAKE_DELAY

        try:
            return [c["WAKE_DELAY"] for c in config["CAMERAS"].values() if c["URI"] == rtsp_uri][0]
        except (TypeError, KeyError, IndexError):
            return DEFAULT_WAKE_DELAY

    async def power_on(self):
        if self.is_powered:
            return

        pin = await self.get_power_pin()
        if pin is None:
            return

        logging.info("Powering cameras on")
        # todo: add awaits when asyncify platform iface
        self.plt_iface.set_do(pin, True)
        self.is_powered = True
        self.start_powered = time.time()

    async def power_off(self):
        if not self.is_powered:
            return

        pin = await self.get_power_pin()
        if pin is None:
            return

        logging.info("Powering cameras off")
        self.plt_iface.set_do(pin, False)
        self.is_powered = False
        self.start_powered = None

    async def handle_cam_done(self, camera_to_handle: str):
        self.active_cameras.remove(camera_to_handle)

        if len(self.active_cameras) == 0:
            await self.power_off()

    def acquire(self, camera_to_manage: str):
        if camera_to_manage in self.active_cameras:
            raise SnapshotInProgress()

        self.active_cameras.add(camera_to_manage)
        return PowerContext(camera_to_manage, self)



from .grpc_stubs import camera_iface_pb2, camera_iface_pb2_grpc
from ..grpc_interface import GRPCInterface
from ...utils import maybe_async
from ...cli.decorators import command as cli_command, annotate_arg

from typing import Optional

class CameraInterface(GRPCInterface):
    stub = camera_iface_pb2_grpc.cameraIfaceStub

    def __init__(self, camera_iface_uri: str = "127.0.0.1:50055", is_async: bool = None):
        super().__init__(camera_iface_uri, is_async)

    @cli_command()
    @maybe_async()
    def get_camera_snapshot(self, camera_uri, channel_name, snapshot_type='mp4', snapshot_length: int = 6):
        req = camera_iface_pb2.SnapshotRequest(
            rtsp_uri=camera_uri,
            snapshot_type=snapshot_type,
            snapshot_length=int(snapshot_length),
            output_channel=channel_name,
        )
        return self.make_request("RunSnapshot", req)

    async def get_camera_snapshot_async(self, camera_uri, channel_name, snapshot_type='mp4', snapshot_length: int = 6):
        req = camera_iface_pb2.SnapshotRequest(
            rtsp_uri=camera_uri,
            snapshot_type=snapshot_type,
            snapshot_length=int(snapshot_length),
            output_channel=channel_name,
        )
        return await self.make_request_async("RunSnapshot", req)

    @cli_command()
    @maybe_async()
    def is_snapshot_running(self):
        resp = self.make_request("IsSnapshotRunning", camera_iface_pb2.IsSnapshotRunningRequest())
        return resp and resp.is_running

    async def is_snapshot_running_async(self):
        resp = await self.make_request_async("IsSnapshotRunning", camera_iface_pb2.IsSnapshotRunningRequest())
        return resp and resp.is_running
    
    @cli_command()
    def test_comms(self, message: str = "Comms Check Message") -> Optional[str]:
        """Test connection by sending a basic echo response to camera interface container.

        Parameters
        ----------
        message : str
            Message to send to camera interface to have echo'd as a response

        Returns
        -------
        str
            The response from camera interface.
        """
        return self.make_request("TestComms", camera_iface_pb2.TestCommsRequest(message=message), response_field="response")


camera_iface = CameraInterface



import argparse
import inspect
import json
import os
import pathlib
import platform
import re
import shutil
import stat
import tarfile
import time
import traceback
import uuid

import logging
import os
import sys
# import absl.logging

from concurrent.futures import ThreadPoolExecutor, as_completed

from datetime import datetime, timedelta
from getpass import getpass
from typing import Optional

import requests

try:
    from fuzzywuzzy import process
    # from simple_term_menu import TerminalMenu
except ImportError:
    process = None
    print("Fuzzywuzzy and/or simple_term_menu not installed. CLI functionality may be limited.")
TerminalMenu = None

from . import parsers
from ..cloud.api import Client, Forbidden, NotFound
from ..cloud.api.channel import Processor, Task
from ..cloud.api.message import Message

from .config import ConfigEntry, ConfigManager, NotSet
from .decorators import command, annotate_arg

from .sub_section import SubSection


DEFAULT_HTTP_DOMAIN = "n1.doover.ngrok.app"
DEFAULT_TCP_DOMAIN = "1.tcp.au.ngrok.io:27735"
TUNNEL_URI_MATCH = re.compile(r"(?P<protocol>(tcp|https))://(?P<host>.*):(?P<port>.*)")
KEY_MATCH = re.compile(r"[0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12}")


class CLI:
    def __init__(self, profile: str = "default", agent: str = None):
        parser = argparse.ArgumentParser(prog="doover", description="Tools for helping with doover.")
        # parser.add_argument("--version", action="version", version=f"doover")
        parser.set_defaults(callback=parser.print_help)

        self.subparser = parser.add_subparsers(dest="subcommand", title="Subcommands")
        self.setup_commands(self.subparser)
        
        self.added_subsections = []

        # to stop circular imports...
        try:
            # fixme: make a [docker] extra feature / package which processors can choose not to install.
            from ..docker.platform import PlatformInterface
            from ..docker.device_agent import DeviceAgentInterface
            from ..docker.tunnel import TunnelInterface
            from ..docker.camera import CameraInterface
            from ..docker.modbus import ModbusInterface
        except ImportError:
            print("Docker interfaces not found. GRPC CLI support will not be available.")
        else:
            self.add_grpc_subsection(SubSection(PlatformInterface, name="platform", description="Interact with a running Platform Interface container"))
            self.add_grpc_subsection(SubSection(DeviceAgentInterface, name="device_agent", description="Interact with a running Device Agent container"))
            self.add_grpc_subsection(SubSection(TunnelInterface, name="tunnel", description="Interact with a running Tunnel Interface container"))
            self.add_grpc_subsection(SubSection(CameraInterface, name="camera", description="Interact with a running Camera Interface container"))
            self.add_grpc_subsection(SubSection(ModbusInterface, name="modbus", description="Interact with a running Modbus Interface container"))
        
        self.args = args = parser.parse_args()

        self.config_manager = ConfigManager()
        self.config_manager.current_profile = getattr(args, "profile", profile)
        self.api: Optional[Client] = None

        if hasattr(args, "agent_id"):
            self.agent_id = args.agent_id if args.agent_id != "default" else agent
        else:
            self.agent_id = None

        if hasattr(args, "agent"):
            self.agent_query = args.agent if args.agent != "default" else None
        else:
            self.agent_query = None
            
        # remove grcp logging while using cli
        os.environ["GRPC_VERBOSITY"] = "ERROR"
        os.environ["GRPC_TRACE"] = ""
        logging.getLogger().setLevel(logging.ERROR)
        sys.stdout.reconfigure(line_buffering=True)

        try:
            passed_args = {
                k: v for k, v in vars(args).items() if k in inspect.signature(args.callback).parameters.keys()
            }
            if ("kwargs" in inspect.signature(args.callback).parameters.keys()):
                passed_args = {
                    k: v for k, v in vars(args).items()
                }
            args.callback(**passed_args)
        except Exception as e:
            self.on_error(e)

    def add_grpc_subsection(self, subsection: SubSection):
        subsection.mount_sub_section(self.subparser)
        self.added_subsections.append(subsection)

    def setup_commands(self, subparser):

        
        for name, func in inspect.getmembers(self, predicate=inspect.ismethod):
            if not getattr(func, "_is_command", False):
                continue

            parser = subparser.add_parser(func._command_name, help=func._command_help)
            parser.set_defaults(callback=func)
            argspec = inspect.signature(func)
            arg_docs = func._command_arg_docs

            for param in argspec.parameters.values():
                kwargs = {"help": arg_docs.get(param.name)}
                param_name = param.name

                if param.default is not inspect.Parameter.empty:
                    kwargs["default"] = param.default
                    kwargs["required"] = False
                    param_name = "--" + param_name

                if param.annotation is parsers.BoolFlag:
                    kwargs["action"] = "store_false" if kwargs["default"] is True else "store_true"
                elif param.annotation is not inspect.Parameter.empty:
                    kwargs["type"] = param.annotation

                parser.add_argument(param_name, **kwargs)

            if func._command_setup_api:
                parser.add_argument("--profile", help="Config profile to use.", default="default")
                parser.add_argument("--agent", help="Agent query string (name or ID) to use for this request.", type=str, default="default")

            parser.add_argument("--enable-traceback", help=argparse.SUPPRESS, default=False, action="store_true")

    def setup_api(self, read: bool = True):
        if read:
            self.config_manager.read()

        config = self.config_manager.current
        if not config:
            raise RuntimeError(f"No configuration found for profile {self.config_manager.current_profile}. "
                               f"Please use a different profile or run `doover login`")

        if self.agent_id is None:
            self.agent_id = config.agent_id

        self.api: Client = Client(
            config.username,
            config.password,
            config.token,
            config.token_expires,
            config.base_url,
            self.agent_id,
            login_callback=self.on_api_login,
        )
        if not (config.token and config.token_expires > datetime.utcnow()):
            self.api.login()

        if self.agent_query is not None:
            self.agent = self.resolve_agent_query(self.agent_query)
            if self.agent is not None:
                self.agent_id = self.api.agent_id = self.agent and self.agent.id
        else:
            self.agent = None

    def resolve_agent_query(self, query_string: str):
        id_match = KEY_MATCH.search(query_string)
        if id_match:
            return self.api.get_agent(id_match.group(0))

        if not (process or TerminalMenu):
            print("Tried to use fuzzy matching without packages installed. "
                  "Please pass an agent ID, or install the extra packages.")
            return

        print("Fetching agents...")
        agents = {a.name: a for a in self.api.get_agent_list()}
        matches = process.extractBests(query_string, agents.keys(), limit=5, score_cutoff=65)
        if len(matches) == 0:
            print(f"Could not resolve agent query: {query_string}. Using default user agent ID.")
            return

        if len(matches) == 1 or len([m for m in matches if m[1] == 100]):
            agent_name, score = matches[0]
            # quick route, no menu required
            print(f"Using agent {agent_name} for API calls. (Query: {query_string}, Score: {score}%)")
            return agents[agent_name]

        options = [f"{m[0]} (Match: {m[1]}%)" for m in matches]
        menu = TerminalMenu(options, title="Select an agent:")
        selected = options[menu.show()]
        agent_name = re.search(r"(.*) \(Match: \d+%\)", selected).group(1)
        print(f"Using agent {agent_name} for API calls. (Query: {query_string})")
        return agents[agent_name]

    def on_api_login(self):
        config: ConfigEntry = self.config_manager.current

        config.agent_id = self.api.agent_id
        config.token = self.api.access_token.token
        config.token_expires = self.api.access_token.expires_at

        self.config_manager.write()

        if self.agent_id is None:
            self.agent_id = config.agent_id

    def on_error(self, exception):
        if isinstance(exception, NotFound):
            print("We couldn't find what you're looking for! Perhaps try a different Agent ID, or check your spelling?")

        elif isinstance(exception, Forbidden):
            print("Uh-oh - you don't have access to that. Perhaps try a different Agent ID, or ask for permissions?")

        elif isinstance(exception, PermissionError):
            print("Looks like you tried to do something to a file you don't have access to. Perhaps try with sudo?")

        else:
            print(f"Hmm... something went wrong: {exception}\n\nPerhaps you can understand more than me?")

        if not self.args.enable_traceback:
            print("Try running with --enable-traceback flag to see the full error.")
        else:
            traceback.print_exc()

    @command(description="Login to your doover account with a username / password")
    def login(self):
        username = input("Please enter your username: ").strip()
        password = getpass("Please enter your password: ").strip()
        base_url = input("Please enter the base API URL: ").strip("%").strip("/")
        profile_name = input("Please enter this profile name (defaults to default): ").strip()
        profile = profile_name if profile_name != "" else "default"

        self.config_manager.create(ConfigEntry(
            profile,
            username=username,
            password=password,
            base_url=base_url,
        ))
        self.config_manager.current_profile = profile

        try:
            self.setup_api(read=False)
            # self.api.login()
        except Exception:
            print("Login failed. Please try again.")
            if self.args.enable_traceback:
                traceback.print_exc()
            return self.login()

        self.config_manager.write()
        print("Login successful.")

    @command()
    def configure_token(self):
        """Configure your doover credentials with a long-lived token"""
        self.configure_token_impl()

    def configure_token_impl(
            self, token: str = None, agent_id: str = None, base_url: str = None, expiry=NotSet, overwrite: bool = False
    ):
        if not token:
            token = input("Please enter your agent token: ").strip()
            # self.config_manager.current.token = token.strip()
        if not agent_id:
            agent_id = input("Please enter your Agent ID: ").strip()
            # self.config_manager.agent_id = agent_id.strip()
        if not base_url:
            base_url = input("Please enter your base API url: ").strip("%").strip("/")
            # self.config.base_url = base_url
        if expiry is NotSet:
            print("This token is intended to be a long-lived token."
                  "I will remind you to reconfigure the token when this expiry is exceeded.")
            expiry_days = input("Please enter the number of days (approximately) until expiration: ")
            try:
                expiry = datetime.utcnow() + timedelta(days=int(expiry_days))
            except ValueError:
                print("I couldn't parse that expiry. I will set it to None which means no expiry.")
                expiry = None

            # self.config.token_expiry = expiry

        profile_name = input("Please enter this profile's name [default]: ")
        profile = profile_name or "default"

        if profile in self.config_manager.entries and not overwrite:
            p = input("There's already a config entry with this profile. Do you want to overwrite it? [y/N]")
            if not p.startswith("y"):
                print("Exitting...")
                return

        self.config_manager.create(ConfigEntry(
            profile, token=token, token_expires=expiry, base_url=base_url, agent_id=agent_id
        ))
        self.config_manager.current_profile = profile

        self.setup_api(read=False)
        try:
            self.api.get_agent(self.agent_id)
        except Forbidden:
            print("Agent token was incorrect. Please try again.")
            return self.configure_token_impl(agent_id=agent_id, base_url=base_url, expiry=expiry, overwrite=True)
        except NotFound:
            print("Agent ID or Base URL was incorrect. Please try again.")
            return self.configure_token_impl(token=token, expiry=expiry, overwrite=True)
        except Exception:
            print("Base URL was incorrect. Please try again.")
            return self.configure_token_impl(token=token, agent_id=agent_id, expiry=expiry, overwrite=True)
        else:
            self.config_manager.write()
            print("Successfully configured doover credentials.")

    @staticmethod
    def format_agent_info(agent):
        fmt = f"""
        Agent Name: {agent.name}
        Agent Type: {agent.type}
        Agent Owner: {agent.owner_org}
        Agent ID: {agent.id}
        """
        return fmt

    @staticmethod
    def format_channel_info(channel):
        fmt = f"""
        Channel Name: {channel.name}
        Channel Type: {str(channel.__class__.__name__)}
        Channel ID: {channel.id}

        Agent ID: {channel.agent_id}
        """
        # Agent Name: {channel.fetch_agent()}

        if isinstance(channel, Task) and channel.processor_id is not None:
            proc = channel.fetch_processor()
            fmt += f"""
        Processor ID: {channel.processor_id}
        Processor Name: {proc.name}
        """
        fmt += f"""
        Aggregate: {json.dumps(channel.aggregate, indent=4)}
        """
        return fmt

    @command(description="List available agents", setup_api=True)
    def get_agent_list(self):
        agents = self.api.get_agent_list()
        for a in agents:
            print(self.format_agent_info(a))

    @command(description="Get channel info", setup_api=True)
    @annotate_arg("channel_name", "Channel name to get info for")
    def get_channel(self, channel_name: str):
        try:
            channel = self.api.get_channel(channel_name)
        except NotFound:
            print(channel_name, self.agent_id)
            channel = self.api.get_channel_named(channel_name, self.agent_id)

        print(self.format_channel_info(channel))

    @command(setup_api=True)
    @annotate_arg("channel_name", "Channel name to create")
    def create_channel(self, channel_name: str):
        """Create new channel"""
        channel = self.api.create_channel(channel_name, self.agent_id)
        print(f"Channel created successfully. ID: {channel.id}")
        print(self.format_channel_info(channel))

    @command(setup_api=True)
    @annotate_arg("task_name", "Task channel name to create.")
    @annotate_arg("processor_name", "Processor name for this task to trigger.")
    def create_task(self, task_name: parsers.task_name, processor_name: parsers.processor_name):
        """Create new task channel."""
        processor = self.api.get_channel_named(processor_name, self.agent_id)
        task = self.api.create_task(task_name, self.agent_id, processor.id)
        print(f"Task created successfully. ID: {task.id}")
        print(self.format_channel_info(task))

    @command(setup_api=True)
    @annotate_arg("task_name", "Task channel name to create.")
    @annotate_arg("package_path", "Path to the  processor package to publish")
    @annotate_arg("channel_name", "[Optional] take the last message from this channel to start the task.")
    @annotate_arg("csv_file", "[Optional] Path to a CSV export of messages to run the task on.")
    @annotate_arg("parallel_processes", "[Optional] Number of parallel processes to run the task with.")
    @annotate_arg("dry_run", "Whether to run the task without invoking it.")
    def invoke_local_task(self,
                            task_name: parsers.task_name,
                            package_path: pathlib.Path,
                            channel_name: Optional[str] = None,
                            csv_file: pathlib.Path = None,
                            parallel_processes: int = None,
                            dry_run: bool = False):
        """Invoke a task locally."""
        task_name = "!" + task_name.lstrip('!')
        task = self.api.get_channel_named(task_name, self.agent_id)
        if not isinstance(task, Task):
            print("That wasn't a task channel. Try again?")
            return
        print(self.format_channel_info(task))

        agent = self.api.get_agent(self.agent_id)

        def run_for_single_message(msg_obj, *args, **kwargs):
            if dry_run:
                return "Dry run successful. Task not invoked."
            msg_dict = msg_obj.to_dict() if msg_obj else None
            task.invoke_locally(
                package_path,
                msg_dict,
                {"deployment_config": agent.deployment_config}
            )
            output = f"Task invoked successfully. Message ID: {msg_obj.id if msg_obj else None}."
            if kwargs:
                output = output + f" Extra kwargs: {kwargs}"
            return output

        if csv_file is not None:
            messages = Message.from_csv_export(self.api, csv_file)
            print(f"Loaded {len(messages)} messages from CSV export.")

            if not parallel_processes or parallel_processes == 1:
                for msg in messages:
                    print(f"\nRunning task for message: {msg.id}, with timestamp: {msg.timestamp}. {messages.index(msg) + 1}/{len(messages)}\n")
                    run_for_single_message(msg)
            else:
                with ThreadPoolExecutor(max_workers=parallel_processes) as executor:
                    futures = [executor.submit(run_for_single_message, msg, task_num=messages.index(msg), total_tasks=len(messages)) for msg in messages]
                    for future in as_completed(futures):
                        print(future.result())

        else:

            msg_obj = None
            if channel_name:
                channel = self.api.get_channel_named(channel_name, self.agent_id)
                msg_obj = channel.last_message

            if not msg_obj:
                print("No message found. running task without a message.")
            else:
                print(f"\nRunning task for message: {msg_obj.id}, with timestamp: {msg_obj.timestamp}\n")
            run_for_single_message(msg_obj)

    @command(setup_api=True)
    def create_processor(self, processor_name: parsers.processor_name):
        """Create new processor channel."""
        processor = self.api.create_processor(processor_name, self.agent_id)
        print(f"Processor created successfully. ID: {processor.id}")
        print(self.format_channel_info(processor))

    @command(setup_api=True)
    @annotate_arg("channel_name", "Channel name to publish to")
    def publish(self, channel_name: str, message: parsers.maybe_json):
        """Publish to a doover channel."""
        try:
            channel = self.api.get_channel_named(channel_name, self.agent_id)
        except NotFound:
            print("Channel name was incorrect. Is it owned by this agent?")
            return

        if isinstance(message, dict):
            print("Successfully loaded message as JSON.")

        channel.publish(message)
        print("Successfully published message.")

    @command(setup_api=True)
    @annotate_arg("channel_name", "Channel name to publish to")
    @annotate_arg("file_path", "Path to the file to publish")
    def publish_file(self, channel_name: str, file_path: pathlib.Path):
        """Publish file to a processor channel."""
        if not file_path.exists():
            print("File path was incorrect.")
            return

        try:
            channel = self.api.get_channel_named(channel_name, self.agent_id)
        except NotFound:
            print("Channel name was incorrect. Is it owned by this agent?")
            return

        channel.update_from_file(file_path)
        print("Successfully published new file.")

    @command(setup_api=True)
    @annotate_arg("processor_name", "Processor channel name to publish to")
    @annotate_arg("package_path", "Path to the package to publish")
    def publish_processor(self, processor_name: parsers.processor_name, package_path: pathlib.Path):
        """Publish processor package to a processor channel."""
        if not package_path.exists():
            print("Package path was incorrect.")
            return

        try:
            channel = self.api.get_channel_named(processor_name, self.agent_id)
        except NotFound:
            print("Channel name was incorrect. Is it owned by this agent?")
            return

        if not isinstance(channel, Processor):
            print("Channel name is not a processor. Try a different name?")
            return

        channel.update_from_package(package_path)
        print("Successfully published new package.")

    @command(setup_api=True)
    @annotate_arg("channel_name", "Channel name to publish to")
    @annotate_arg("poll_rate", "Frequency to check for new messages (in seconds)")
    def follow_channel(self, channel_name: str, poll_rate: int = 5):
        """Follow aggregate of a doover channel"""
        channel = self.api.get_channel_named(channel_name, self.agent_id)
        print(self.format_channel_info(channel))

        while True:
            old_aggregate = channel.aggregate
            channel.update()
            if channel.aggregate != old_aggregate:
                print(channel.aggregate)

            time.sleep(poll_rate)

    @command(setup_api=True)
    @annotate_arg("task_name", "Task name to add the subscription to")
    @annotate_arg("channel_name", "Channel name to subscribe to")
    def subscribe_channel(self, task_name: parsers.task_name, channel_name: str):
        """Add a channel to a task's subscriptions."""
        task = self.api.get_channel_named(task_name, self.agent_id)
        if not isinstance(task, Task):
            print("That wasn't a task channel. Try again?")
            return

        channel = self.api.get_channel_named(channel_name, self.agent_id)
        task.subscribe_to_channel(channel.id)
        print(f"Successfully added {channel_name} to {task.name}'s subscriptions.")

    @command(setup_api=True)
    @annotate_arg("task_name", "Task name to remove the subscription from")
    @annotate_arg("channel_name", "Channel name to unsubscribe from")
    def unsubscribe_channel(self, task_name: parsers.task_name, channel_name: str):
        """Remove a channel to a task's subscriptions."""
        task = self.api.get_channel_named(task_name, self.agent_id)
        if not isinstance(task, Task):
            print("That wasn't a task channel. Try again?")
            return

        channel = self.api.get_channel_named(channel_name, self.agent_id)
        task.unsubscribe_from_channel(channel.id)
        print(f"Successfully removed {channel_name} from {task.name}'s subscriptions.")

    @command(setup_api=True)
    @annotate_arg("config_file", "Deployment config file to use. This is usually a doover_config.json file.")
    def deploy_config(self, config_file: pathlib.Path):
        """Deploy a doover config file to the site."""
        if not config_file.exists():
            print("Config file not found.")
            return

        parent_dir = os.path.dirname(config_file)

        with open(config_file, "r") as config_file:
            data = json.loads(config_file.read())

        print("Read config file.")

        proc_deploy_data = data.get("processor_deployments")
        if proc_deploy_data:
            for processor_data in proc_deploy_data.get("processors", []):
                processor = self.api.create_processor(processor_data["name"], self.agent_id)
                processor.update_from_package(os.path.join(parent_dir, processor_data["processor_package_dir"]))
                processor.update()
                print(f"Created or updated processor {processor.name} with processor data length: {len(processor.aggregate)}")

            for task_data in proc_deploy_data.get("tasks", []):
                processor = self.api.get_channel_named(task_data["processor_name"], self.agent_id)
                task = self.api.create_task(task_data["name"], self.agent_id, processor.id)
                task.publish(task_data["task_config"])
                print(f"Created or updated task {task.name}, and deployed new config.")

                for subscription in task_data.get("subscriptions", []):
                    channel = self.api.create_channel(subscription["channel_name"], self.agent_id)
                    if subscription["is_active"] is True:
                        task.subscribe_to_channel(channel.id)
                        print(f"Added {channel.name} as a subscription to task {task.name}.")
                    else:
                        task.unsubscribe_from_channel(channel.id)
                        print(f"Removed {channel.name} as a subscription from task {task.name}.")

        file_deploy_data = data.get("file_deployments")
        if file_deploy_data:
            for entry in file_deploy_data.get("files", []):
                channel = self.api.create_channel(entry["name"], self.agent_id)
                mime_type = entry.get("mime_type", None)
                channel.update_from_file(os.path.join(parent_dir, entry["file_dir"]), mime_type)
                print(f"Published file to {channel.name}")

        for entry in data.get("deployment_channel_messages", []):
            channel = self.api.create_channel(entry["channel_name"], self.agent_id)
            save_log = entry.get("save_log", True)
            channel.publish(entry["channel_message"], save_log=save_log)
            print(f"Published message to {channel.name}")

        print("Successfully deployed config.")

    @staticmethod
    def _get_ip():
        return requests.get("https://api.ipify.org").text

    def _create_tunnel(
        self, hostname: str, port: int,
        protocol: str, timeout: int,
        restrict_cidr: bool = True
    ):
        tunnels = self.api.get_tunnels(self.agent_id)
        if restrict_cidr:
            my_ip = self._get_ip()

        tunnel = None
        for t in tunnels["tunnels"]:
            if t["hostname"] == hostname and t["port"] == port:
                tunnel = t
                break

        if tunnel:
            if tunnel["timeout"] != timeout or tunnel["ip_restricted"] != restrict_cidr or (restrict_cidr and my_ip not in tunnel["ip_whitelist"]):
                print("Existing tunnel found, but with different settings. Editting tunnel...")
                self.api.patch_tunnel(
                    tunnel["key"],
                    timeout=timeout,
                    ip_restricted=restrict_cidr,
                    ip_whitelist=[my_ip] if restrict_cidr else [],
                )
            print(f"Found existing tunnel: {tunnel['endpoint']}...")
            return tunnel

        print("No tunnel found. Opening tunnel... Please wait...")
        tunnel = self.api.create_tunnel(
            self.agent_id,
            hostname=hostname,
            port=port,
            protocol=protocol,
            name=f"{hostname}:{port}",
            ip_restricted=restrict_cidr,
            is_favourite=True,
            ip_whitelist=[my_ip] if restrict_cidr else [],
            timeout=timeout,
        )

        return tunnel

    def _activate_tunnel(self, tunnel_id, wait_for_open: bool = True):
        self.api.activate_tunnel(tunnel_id)
        print(f"Activated tunnel {tunnel_id}.")

        if wait_for_open:
            print("Waiting for tunnel to open...")
            while True:
                tunnel = self.api.get_tunnel(tunnel_id)
                if tunnel["is_active"]:
                    print("Tunnel is open.")
                    break

                time.sleep(1)

    @command(description="Open an SSH tunnel for a doover agent", setup_api=True)
    def open_ssh_tunnel(self, timeout: int = 15, restrict_cidr: bool = True):
        tunnel = self._create_tunnel("127.0.0.1", 22, "tcp", timeout, restrict_cidr)
        print(tunnel)
        if not tunnel["is_active"]:
            self._activate_tunnel(tunnel["key"], True)

        host, port = tunnel["endpoint"].split(":")

        username = input("Please enter your SSH username: ")

        print(f"Opening SSH session with host: {host}, port: {port}, username: {username}...")
        os.execl("/usr/bin/ssh", "ssh", f"{username}@{host}", "-p", port)

    @command(description="Open an arbitrary tunnel for a doover agent", setup_api=True)
    def open_tunnel(self, address: str, protocol: str = "http", timeout: int = 15, restrict_cidr: bool = True):
        host, port = address.split(":")
        self._create_tunnel(host, int(port), protocol, timeout, restrict_cidr)

    @command(description="Close all tunnels for a doover agent", setup_api=True)
    def close_all_tunnels(self):
        channel = self.api.get_channel_named("tunnels", self.agent_id)
        channel.publish({"to_close": channel.aggregate["open"]})
        print("Successfully closed all tunnels.")

    def _activate_deactivate_tunnel(self, tunnel_id: str = None, activate: bool = True):
        action = self.api.activate_tunnel if activate else self.api.deactivate_tunnel
        action_word = "activate" if activate else "deactivate"

        if tunnel_id:
            action(tunnel_id)
            print(f"Successfully {'activated' if activate else 'deactivated'} tunnel.")
            return

        tunnels = self.api.get_tunnels(self.agent_id)
        if not tunnels.get("tunnels", []):
            print("No tunnels found.")
            return

        options = [f"{tunnel['name']} ({tunnel['endpoint']})" for tunnel in tunnels["tunnels"]]
        menu = TerminalMenu(options, title="Select an agent:")
        tunnel = tunnels["tunnels"][menu.show()]
        action(tunnel['key'])
        print(f"Successfully {action_word} tunnel.")

    @command(description="Get tunnels for an agent", setup_api=True)
    def get_tunnels(self):
        tunnels = self.api.get_tunnels(self.agent_id)
        for tunnel in tunnels["tunnels"]:
            print(f"{tunnel['name']} ({tunnel['endpoint']}) - {'Active' if tunnel['is_active'] else 'Inactive'}")

    @command(description="Activate a tunnel", setup_api=True)
    def activate_tunnel(self, tunnel_id: str = None):
        self._activate_deactivate_tunnel(tunnel_id, activate=True)

    @command(description="Deactivate a tunnel", setup_api=True)
    def deactivate_tunnel(self, tunnel_id: str = None):
        self._activate_deactivate_tunnel(tunnel_id, activate=False)

    # @command(description="Create new tunnel endpoints for an agent", setup_api=True)
    # def create_tunnel_endpoints(self, endpoint_type: str = "tcp", amount: int = 1):
    #     if endpoint_type not in ("tcp", "http"):
    #         print("Endpoint type must be either tcp or http.")
    #         return
    #
    #     if amount < 1:
    #         print("Amount must be a number greater than or equal to 1.")
    #         return
    #
    #     data = self.api.create_tunnel_endpoints(self.agent_id, endpoint_type, amount)
    #     for d in data:
    #         print(f"Created new {endpoint_type} endpoint: {d}")
    #
    #     channel = self.api.get_channel_named("tunnels", self.agent_id)
    #     key = f"{endpoint_type}_endpoints"
    #     channel.publish({key: channel.aggregate.get(key, []) + data})

    @command(description="List ngrok tunnel endpoints for an agent", setup_api=True)
    def list_tunnel_endpoints(self):
        print("#################\nDEPRECATED##################\n\nYou're probably after the `get_tunnels` command.")
        tcp = self.api.get_tunnel_endpoints(self.agent_id, "tcp")
        http = self.api.get_tunnel_endpoints(self.agent_id, "http")

        print(f"HTTP Endpoints\n==============\n" + '\n'.join(http))
        print(f"TCP Endpoints\n=============\n" + '\n'.join(tcp))


    def main(self):
        pass



#!/usr/bin/env python3

import asyncio
import logging
import math
from typing import Optional


class LocationManager:
    def __init__(self, app):
        self.app = app
        self.accuracy_threshold: Optional[float] = None
        self.distance_threshold: Optional[float] = None
        self.location_update_freq_secs: Optional[float] = None
        self.last_published_location = None
        self.last_location_update_time: Optional[float] = None
        self.is_active = False

    async def fetch_config(self):
        config = await self.app.get_config_async(key_filter="LOCATION_MANAGER", wait=True)
        if not config:
            config = {}
            ## Enable LocationManager by default

        if config.get("ENABLED", True) is False:
            logging.info("LocationManager is disabled. Skipping setup.")
            return

        self.accuracy_threshold = config.get("ACCURACY_THRESHOLD", 10)
        self.distance_threshold = config.get("DISTANCE_THRESHOLD", 15)
        self.location_update_freq_secs = config.get("LOCATION_UPDATE_FREQ_SECS", 15)

        if not all(isinstance(value, (int, float)) for value in [
            self.accuracy_threshold, self.distance_threshold, self.location_update_freq_secs
        ]):
            logging.warning("Invalid configuration for LocationManager. Disabling Location Management.")
            self.accuracy_threshold = None
            self.distance_threshold = None
            self.location_update_freq_secs = None
            return

        logging.info(f"LocationManager configured with accuracy_threshold={self.accuracy_threshold}, "
                     f"distance_threshold={self.distance_threshold}, "
                     f"location_update_freq_secs={self.location_update_freq_secs}.")
        self.is_active = True

    async def fetch_location(self) -> Optional[dict]:
        try:
            location = await self.app.platform_iface.get_location_async()
            if not location:
                logging.warning("Failed to fetch location.")
                return None

            ## Transform the location data to a dictionary
            location = {
                "lat": location.latitude,
                "long": location.longitude,
                "alt": location.altitude_m,
                "accuracy": location.accuracy_m,
            }

            logging.debug(f"Fetched location: {location}")
            return location
        except Exception as e:
            logging.error(f"Error fetching location: {e}")
            return None

    def calculate_distance(self, loc1: dict, loc2: dict) -> float:
        """
        Calculate the distance between two locations using the haversine formula.

        :param loc1: First location as a dictionary with 'lat' and 'long'.
        :param loc2: Second location as a dictionary with 'lat' and 'long'.
        :return: Distance in meters.
        """
        R = 6371000  # Earth radius in meters
        lat1, lon1 = math.radians(loc1['lat']), math.radians(loc1['long'])
        lat2, lon2 = math.radians(loc2['lat']), math.radians(loc2['long'])

        dlat = lat2 - lat1
        dlon = lon2 - lon1

        a = math.sin(dlat / 2) ** 2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon / 2) ** 2
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
        return R * c

    async def subscribe_to_location_channel(self):
        self.app.get_agent_iface().add_subscription("location", self.handle_location_channel_update)

    async def handle_location_channel_update(self, channel_name, aggregate):
        logging.debug(f"Received update from {channel_name}: {aggregate}")
        if aggregate and channel_name == "location":
            recv_location = aggregate
            if isinstance(aggregate, str):
                recv_location = {}
                
            self.last_published_location = {
                "lat": recv_location.get("lat", None),
                "long": recv_location.get("long", None),
                "alt": recv_location.get("alt", None),
                "accuracy": recv_location.get("accuracy", None),
            }

    async def publish_location(self, location: dict) -> bool:
        try:
            success = await self.app.get_agent_iface().publish_to_channel_async("location", location)
            return success
        except Exception as e:
            logging.error(f"Error publishing location: {e}")
            return False

    async def setup(self):
        logging.info("Setting up LocationManager...")
        await self.fetch_config()
        if self.is_active:
            await self.subscribe_to_location_channel()

    async def main_loop(self):
        
        if not self.is_active:
            logging.info("LocationManager is inactive. Skipping main loop cycle.")
            return

        current_time = asyncio.get_event_loop().time()

        # Check if the location update frequency interval has passed
        if self.last_location_update_time is not None and \
           (current_time - self.last_location_update_time < self.location_update_freq_secs):
            logging.debug("Location update frequency interval not reached. Skipping.")
            return

        # Update the last location update time
        self.last_location_update_time = current_time

        # Fetch the current location
        location = await self.fetch_location()
        if not location:
            logging.debug("Location is null, skipping update")
            return

        accuracy = location.get("accuracy", float("inf"))
        if accuracy > self.accuracy_threshold:
            logging.debug(f"Location accuracy {accuracy} exceeds threshold {self.accuracy_threshold}. Skipping publish.")
            return

        if self.last_published_location:
            distance = self.calculate_distance(self.last_published_location, location)
            if distance < self.distance_threshold:
                logging.debug(f"Location change ({distance}m) is below threshold {self.distance_threshold}m. Skipping publish.")
                return

        # Publish the new location
        success = await self.publish_location(location)
        if success:
            logging.info(f"Published location: {location}")
            self.last_published_location = location
        else:
            logging.error("Failed to publish location.")


def find_object_with_key(obj, key_to_find):
    """
    Iteratively searches through a dictionary (JSON object) and returns the value for the specified key.
    :param obj: The JSON object (dictionary) to search through.
    :param key_to_find: The key to search for.
    :return: The object containing the key, or None if the key is not found.
    """
    stack = [obj]

    while stack:
        current = stack.pop()

        if isinstance(current, dict):
            if key_to_find in current:
                return current[key_to_find]

            for key in current:
                stack.append(current[key])

    return None


def find_path_to_key(obj, key_to_find):
    """
    Iteratively searches through a dictionary (JSON object) and returns the path to the specified key.
    :param obj: The JSON object (dictionary) to search through.
    :param key_to_find: The key to search for.
    :return: The path to the key, or None if the key is not found.
    """
    stack = [{'current': obj, 'path': ''}]

    while stack:
        current_item = stack.pop()
        current, path = current_item['current'], current_item['path']

        if isinstance(current, dict):
            if key_to_find in current:
                return f"{path}.{key_to_find}" if path else key_to_find

            for key in current:
                new_path = f"{path}.{key}" if path else key
                stack.append({'current': current[key], 'path': new_path})

    return None


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import tunnel_iface_pb2 as grpc__stubs_dot_tunnel__iface__pb2

GRPC_GENERATED_VERSION = '1.65.1'
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = '1.66.0'
SCHEDULED_RELEASE_DATE = 'August 6, 2024'
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in grpc_stubs/tunnel_iface_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
        + f' This warning will become an error in {EXPECTED_ERROR_RELEASE},'
        + f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',
        RuntimeWarning
    )


class TunnelInterfaceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.TestComms = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/TestComms',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.TestCommsRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.TestCommsResponse.FromString,
                _registered_method=True)
        self.OpenTunnel = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/OpenTunnel',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelResponse.FromString,
                _registered_method=True)
        self.CloseTunnel = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/CloseTunnel',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelResponse.FromString,
                _registered_method=True)
        self.CloseAllTunnels = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/CloseAllTunnels',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsResponse.FromString,
                _registered_method=True)
        self.GetTunnel = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/GetTunnel',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.GetTunnelRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.GetTunnelResponse.FromString,
                _registered_method=True)
        self.SyncTunnels = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/SyncTunnels',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsResponse.FromString,
                _registered_method=True)
        self.ListTunnels = channel.unary_unary(
                '/tunnel_iface.TunnelInterface/ListTunnels',
                request_serializer=grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsRequest.SerializeToString,
                response_deserializer=grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsResponse.FromString,
                _registered_method=True)


class TunnelInterfaceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def TestComms(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenTunnel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseTunnel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseAllTunnels(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTunnel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SyncTunnels(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTunnels(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TunnelInterfaceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'TestComms': grpc.unary_unary_rpc_method_handler(
                    servicer.TestComms,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.TestCommsRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.TestCommsResponse.SerializeToString,
            ),
            'OpenTunnel': grpc.unary_unary_rpc_method_handler(
                    servicer.OpenTunnel,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelResponse.SerializeToString,
            ),
            'CloseTunnel': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseTunnel,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelResponse.SerializeToString,
            ),
            'CloseAllTunnels': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseAllTunnels,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsResponse.SerializeToString,
            ),
            'GetTunnel': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTunnel,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.GetTunnelRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.GetTunnelResponse.SerializeToString,
            ),
            'SyncTunnels': grpc.unary_unary_rpc_method_handler(
                    servicer.SyncTunnels,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsResponse.SerializeToString,
            ),
            'ListTunnels': grpc.unary_unary_rpc_method_handler(
                    servicer.ListTunnels,
                    request_deserializer=grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsRequest.FromString,
                    response_serializer=grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tunnel_iface.TunnelInterface', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('tunnel_iface.TunnelInterface', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class TunnelInterface(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def TestComms(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/TestComms',
            grpc__stubs_dot_tunnel__iface__pb2.TestCommsRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.TestCommsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def OpenTunnel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/OpenTunnel',
            grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.OpenTunnelResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CloseTunnel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/CloseTunnel',
            grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.CloseTunnelResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CloseAllTunnels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/CloseAllTunnels',
            grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.CloseAllTunnelsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetTunnel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/GetTunnel',
            grpc__stubs_dot_tunnel__iface__pb2.GetTunnelRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.GetTunnelResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SyncTunnels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/SyncTunnels',
            grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.SyncTunnelsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListTunnels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/tunnel_iface.TunnelInterface/ListTunnels',
            grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsRequest.SerializeToString,
            grpc__stubs_dot_tunnel__iface__pb2.ListTunnelsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)
